/*
 * Copyright (c) 2008 Jesper Karlsson & David Gustafsson
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <gtk/gtk.h>
#include <libxfce4ui/libxfce4ui.h>
#include <string.h>
#include <glib.h>

#include "xfbib-entry-edit-dialog.h"
#include "xfbib-multiple-input.h"
#include "xfbib-bibtex-field.h"
#include "xfbib-bibtex-entry.h"
#include "xfbib-state.h"

static struct {
	gchar *text;
}xfbib_type_constants[] = {
	{"Article"},
	{"Book"},
	{"Booklet"},
	{"Conference"},
	{"Inbook"},
	{"Incollection"},
	{"Inproceedings"},
	{"Manual"},
	{"Mastersthesis"},
	{"Misc"},
	{"PhdThesis"},
	{"Proceedings"},
	{"Techreport"},
	{"Unpublished"},
	{NULL}
};

struct input {
	GtkWidget *label;	/* Label and NULL objects */
	GtkWidget *input;	/* Entry and multiple inputs */
	GtkWidget *button;
	GList *gl;	/* Used for compound strings */
};

struct _XfbibEntryEditDialog
{
	XfceTitledDialog parent;
	GtkWidget *type_label;
	GtkWidget *type_combo_box;
	GtkWidget *key_label;
	GtkWidget *key_entry;
	GtkWidget *key_button;
	struct input inputs[XFBIB_BIBTEX_FIELD_N_FIELDS];
	GtkWidget *note_boxes[5];
	GtkWidget *notebook;
};

typedef struct _XfbibEntryEditDialogClass
{
	XfceTitledDialogClass parent;
} XfbibEntryEditDialogClass;

static void xfbib_entry_edit_dialog_class_init(XfbibEntryEditDialogClass *klass);

static void xfbib_entry_edit_dialog_init(XfbibEntryEditDialog *instance);
static void xfbib_entry_edit_dialog_finalize(GObject *obj);

static GObjectClass *xfbib_entry_edit_dialog_parent_class = NULL;

static void
cb_generate_clicked (GtkButton *button, gpointer user_data)
{
	g_print("cb_generate_clicked\n");
	/* Generate BibTeX key */
	/* Suggested format: Last name of first author, year of publication, first word in title appended together */
	GString *str;
	const gchar *lastname, *title;
	gchar **split;
	char *tmp;
	str = g_string_new("");

	lastname = xfbib_multiple_input_get_nth_row_ith_column(XFBIB_MULTIPLE_INPUT (XFBIB_ENTRY_EDIT_DIALOG (user_data)->inputs[XFBIB_BIBTEX_FIELD_AUTHOR].input),0,0);
	if(lastname != NULL) {
		tmp = strrchr(lastname, ' ') + 1;
		g_string_append(str, tmp);
	}

	g_string_append(str, (gtk_entry_get_text(GTK_ENTRY (XFBIB_ENTRY_EDIT_DIALOG (user_data)->inputs[XFBIB_BIBTEX_FIELD_YEAR].input)) + 2));

	title = gtk_entry_get_text(GTK_ENTRY (XFBIB_ENTRY_EDIT_DIALOG (user_data)->inputs[XFBIB_BIBTEX_FIELD_TITLE].input));
	split = g_strsplit(title, " ", 2);
	title = split[0];
	g_string_append(str, title);

	gtk_entry_set_text(GTK_ENTRY (XFBIB_ENTRY_EDIT_DIALOG (user_data)->key_entry), str->str);
	g_string_free(str, TRUE);
}

static void
cb_edit_compound_string_clicked (GtkButton *button, struct input *inputs)
{
	g_print("cb_compound_string_clicked\n");

	GtkWidget *multiple_input;
	const gchar *str;

	GtkWidget *dialog = xfce_titled_dialog_new_with_buttons(_("Edit compound string"),
                                                    NULL,
                                                    GTK_DIALOG_MODAL & GTK_DIALOG_DESTROY_WITH_PARENT,
                                                    NULL);
	gtk_window_set_icon_name(GTK_WINDOW(dialog), "xfbib");
	
	/* Add OK and cancel button */
	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_OK, GTK_RESPONSE_OK);
	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
	
	multiple_input = xfbib_multiple_input_new(_("Text/String key"), _("Row can contain either text or a BibTeX string key"));
	if(GTK_WIDGET_IS_SENSITIVE(inputs->input)) {
		if(strcmp(gtk_entry_get_text(GTK_ENTRY(inputs->input)), "") != 0) {
			xfbib_multiple_input_set_text(XFBIB_MULTIPLE_INPUT(multiple_input), gtk_entry_get_text(GTK_ENTRY(inputs->input)));
		}
	} else {
		xfbib_multiple_input_set(XFBIB_MULTIPLE_INPUT(multiple_input), inputs->gl);
	}

	gtk_box_pack_start(GTK_BOX (GTK_DIALOG(dialog)->vbox), multiple_input, TRUE, TRUE, 6); 
	gtk_widget_show_all(GTK_WIDGET(dialog));

	if(gtk_dialog_run(GTK_DIALOG (dialog)) == GTK_RESPONSE_OK) {
		str = xfbib_multiple_input_get_str(XFBIB_MULTIPLE_INPUT (multiple_input));
		if(str != NULL) {
			gtk_widget_set_sensitive(GTK_WIDGET(inputs->input), TRUE);
			gtk_entry_set_text(GTK_ENTRY(inputs->input), str);
		} else {
			gtk_widget_set_sensitive(GTK_WIDGET(inputs->input), FALSE);
			gtk_entry_set_text(GTK_ENTRY(inputs->input), "");
			inputs->gl = xfbib_multiple_input_get(XFBIB_MULTIPLE_INPUT (multiple_input));
		}
	}

	gtk_widget_destroy(dialog);
}

static GtkWidget*
xfbib_get_image (gchar *filename)
{
	GtkWidget *image;
	GString *path;
	GdkPixbuf *pixbuf = NULL;

	path = g_string_new("");
	g_string_append(path, DATADIR);
	g_string_append(path, "/pixmaps/xfbib/");
	g_string_append(path, filename);

	/* TODO: Fix the error handling */
	pixbuf = gdk_pixbuf_new_from_file(path->str, NULL);
	g_string_free(path, TRUE);

	if(pixbuf == NULL){
		g_warning(_("Icon missing: %s"), filename);
		image = gtk_image_new_from_stock(GTK_STOCK_MISSING_IMAGE, GTK_ICON_SIZE_BUTTON);
	}else{
		image = gtk_image_new_from_pixbuf(pixbuf);
	}

	return image;
}

struct field_properties xfbib_bibtex_field_constants[] = {
	{"Address", ("Address of publisher")},
	{"Annote", ("Annotation for annotated bibliography styles")},
	{"Author", ("Name(s) of the author(s), separated by 'and' if more than one")},
	{"Booktitle", ("Title of the book, if only part of it is being cited")},
	{"Chapter", ("Chapter number")},
	{"Crossref", ("Citation key of the cross-referenced entry")},
	{"Edition", ("Edition of the book (such as \"first\" or \"second\")")},
	{"Editor", ("Name(s) of the editor(s), separated by 'and' if more than one")},
	{"E-print", ("Specification of electronic publication")},
	{"HowPublished", ("Publishing method if the method is nonstandard")},
	{"Institution", ("Institution that was involved in the publishing")},
	{"Journal", ("Journal or magazine in which the work was published")},
	{"Key", ("Hidden field used for specifying or overriding the alphabetical order of entries")},
	{"Month", ("Month of publication or creation if unpublished")},
	{"Note", ("Miscellaneous extra information")},
	{"Number", ("Number of journal, magazine, or tech-report")},
	{"Organization", ("Sponsor of the conference")},
	{"Pages", ("Page numbers separated by commas or double-hyphens")},
	{"Publisher", ("Name of publisher")},
	{"School", ("School where thesis was written")},
	{"Series", ("Series of books in which the book was published")},
	{"Title", ("Title of the work")},
	{"Type", ("Type of technical report")},
	{"URL", ("Internet address")},
	{"Volume", ("Number of the volume")},
	{"Year", ("Year of publication or creation if unpublished")}
};

GType
xfbib_entry_edit_dialog_get_type (void)
{
	static GType type = 0;
	if (type == 0) {
		static const GTypeInfo info = {
			sizeof (XfbibEntryEditDialogClass),
			NULL,   /* base_init */
			NULL,   /* base_finalize */
			(GClassInitFunc) xfbib_entry_edit_dialog_class_init,   /* class_init */
			NULL,   /* class_finalize */
			NULL,   /* class_data */
			sizeof (XfbibEntryEditDialog),
			0,      /* n_preallocs */
			(GInstanceInitFunc) xfbib_entry_edit_dialog_init    /* instance_init */
		};
		type = g_type_register_static (XFCE_TYPE_TITLED_DIALOG, "XfbibEntryEditDialog", &info, 0);
	}
	return type;
}

static void
xfbib_entry_edit_dialog_class_init(XfbibEntryEditDialogClass *klass)
{
	GObjectClass *object_class = (GObjectClass *)klass;

	xfbib_entry_edit_dialog_parent_class = g_type_class_peek_parent (klass);

	object_class->finalize = xfbib_entry_edit_dialog_finalize;
}

static void
xfbib_entry_edit_dialog_init(XfbibEntryEditDialog *instance)
{
	gint i, row[5] = {0, 0, 0, 0, 0}, box = -1;
	GtkWidget *top_box;

	instance->type_label = gtk_label_new(_("Type:"));
	instance->type_combo_box = gtk_combo_box_new_text();
	for (i = 0; xfbib_type_constants[i].text != NULL; i++) {
		gtk_combo_box_append_text(GTK_COMBO_BOX(instance->type_combo_box),
				xfbib_type_constants[i].text);
	}

	instance->key_label = gtk_label_new(_("Key:"));
	instance->key_entry = gtk_entry_new();
	instance->key_button = gtk_button_new();
	gtk_container_add(GTK_CONTAINER(instance->key_button), xfbib_get_image("xfbib-generate.png"));
	g_signal_connect(G_OBJECT (instance->key_button), "clicked", G_CALLBACK (cb_generate_clicked), instance);

	top_box = gtk_hbox_new(FALSE, 12);

	gtk_container_set_border_width(GTK_CONTAINER (top_box), 6);

	gtk_box_pack_start(GTK_BOX(top_box), instance->type_label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(top_box), instance->type_combo_box, FALSE, FALSE, 0);

	gtk_box_pack_end(GTK_BOX(top_box), instance->key_button, FALSE, FALSE, 0);
	gtk_box_pack_end(GTK_BOX(top_box), instance->key_entry, TRUE, TRUE, 0);
	gtk_box_pack_end(GTK_BOX(top_box), instance->key_label, FALSE, FALSE, 0);

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (instance)->vbox), top_box, FALSE, FALSE, 0);

	gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(instance)->vbox), 0);	// 0 due to top_box border width

	for (i = 0; i < 5; i++) {
		instance->note_boxes[i] = gtk_table_new(1, 1, FALSE);
		gtk_table_set_row_spacings(GTK_TABLE(instance->note_boxes[i]), 6);
		gtk_table_set_col_spacings(GTK_TABLE(instance->note_boxes[i]), 12);
		gtk_container_set_border_width(GTK_CONTAINER (instance->note_boxes[i]), 6);
	}

	for (i = 0; i < XFBIB_BIBTEX_FIELD_N_FIELDS; i++) {
		/* Create the Input object */
		if (i == XFBIB_BIBTEX_FIELD_AUTHOR || i == XFBIB_BIBTEX_FIELD_EDITOR) {
			instance->inputs[i].input = xfbib_multiple_input_new(xfbib_bibtex_field_constants[i].label, xfbib_bibtex_field_constants[i].tooltip);
		} else {
			instance->inputs[i].label = gtk_label_new(xfbib_bibtex_field_constants[i].label);
			gtk_misc_set_alignment(GTK_MISC(instance->inputs[i].label), 0, 0.5);

			instance->inputs[i].input = gtk_entry_new();
			gtk_widget_set_tooltip_text(instance->inputs[i].input, xfbib_bibtex_field_constants[i].tooltip);

			instance->inputs[i].button = gtk_button_new();
			gtk_container_add(GTK_CONTAINER(instance->inputs[i].button), xfbib_get_image("xfbib-compound-string.png"));
			g_signal_connect(G_OBJECT (instance->inputs[i].button), "clicked", G_CALLBACK (cb_edit_compound_string_clicked), &(instance->inputs[i]));
		}

		/* Add the input object to the correct box */
		switch (i) {
			case XFBIB_BIBTEX_FIELD_INSTITUTION:
			case XFBIB_BIBTEX_FIELD_ORGANIZATION:
			case XFBIB_BIBTEX_FIELD_SCHOOL:
			case XFBIB_BIBTEX_FIELD_TITLE:
			case XFBIB_BIBTEX_FIELD_TYPE:
				box = 0;
				break;
			case XFBIB_BIBTEX_FIELD_AUTHOR:
			case XFBIB_BIBTEX_FIELD_EDITOR:
				box = 1;
				break;
			case XFBIB_BIBTEX_FIELD_BOOKTITLE:
			case XFBIB_BIBTEX_FIELD_CHAPTER:
			case XFBIB_BIBTEX_FIELD_EDITION:
			case XFBIB_BIBTEX_FIELD_MONTH:
			case XFBIB_BIBTEX_FIELD_NUMBER:
			case XFBIB_BIBTEX_FIELD_PAGES:
			case XFBIB_BIBTEX_FIELD_VOLUME:
			case XFBIB_BIBTEX_FIELD_YEAR:
				box = 2;
				break;
			case XFBIB_BIBTEX_FIELD_ADDRESS:
			case XFBIB_BIBTEX_FIELD_HOWPUBLISHED:
			case XFBIB_BIBTEX_FIELD_JOURNAL:
			case XFBIB_BIBTEX_FIELD_PUBLISHER:
			case XFBIB_BIBTEX_FIELD_SERIES:
				box = 3;
				break;
			case XFBIB_BIBTEX_FIELD_ANNOTE:
			case XFBIB_BIBTEX_FIELD_CROSSREF:
			case XFBIB_BIBTEX_FIELD_EPRINT:
			case XFBIB_BIBTEX_FIELD_KEY:
			case XFBIB_BIBTEX_FIELD_NOTE:
			case XFBIB_BIBTEX_FIELD_URL:
				box = 4;
				break;
		}
		if (i == XFBIB_BIBTEX_FIELD_AUTHOR || i == XFBIB_BIBTEX_FIELD_EDITOR) {
			gtk_table_attach_defaults(GTK_TABLE(instance->note_boxes[box]), instance->inputs[i].input, row[box], row[box]+1, 0, 1);
		} else {
			gtk_table_attach(GTK_TABLE(instance->note_boxes[box]), instance->inputs[i].label, 0, 1, row[box], row[box]+1, GTK_FILL, GTK_FILL, 0, 0);
			gtk_table_attach(GTK_TABLE(instance->note_boxes[box]), instance->inputs[i].input, 1, 2, row[box], row[box]+1, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
			gtk_table_attach(GTK_TABLE(instance->note_boxes[box]), instance->inputs[i].button, 2, 3, row[box], row[box]+1, GTK_FILL, GTK_FILL, 0, 0);
		}
		row[box]++;
	}

	instance->notebook = gtk_notebook_new ();
	gtk_container_set_border_width(GTK_CONTAINER (instance->notebook), 6);
	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (instance->notebook), GTK_POS_TOP);
	gtk_notebook_append_page (GTK_NOTEBOOK (instance->notebook), instance->note_boxes[0], gtk_label_new(_("Title")));
	gtk_notebook_append_page (GTK_NOTEBOOK (instance->notebook), instance->note_boxes[1], gtk_label_new(_("Author/Editor")));
	gtk_notebook_append_page (GTK_NOTEBOOK (instance->notebook), instance->note_boxes[2], gtk_label_new(_("Publication")));
	gtk_notebook_append_page (GTK_NOTEBOOK (instance->notebook), instance->note_boxes[3], gtk_label_new(_("Publisher")));
	gtk_notebook_append_page (GTK_NOTEBOOK (instance->notebook), instance->note_boxes[4], gtk_label_new(_("Misc")));

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (instance)->vbox), instance->notebook, TRUE, TRUE, 0);

	/* Add OK and cancel button */
	gtk_dialog_add_button (GTK_DIALOG (instance), GTK_STOCK_OK, GTK_RESPONSE_OK);
	gtk_dialog_add_button (GTK_DIALOG (instance), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);

	gtk_dialog_set_default_response (GTK_DIALOG (instance), GTK_RESPONSE_CANCEL);
	gtk_window_set_icon_name(GTK_WINDOW(instance), "xfbib");
	gtk_widget_show_all(GTK_WIDGET(instance));
}

static void
xfbib_entry_edit_dialog_finalize(GObject *obj)
{
	G_OBJECT_CLASS(xfbib_entry_edit_dialog_parent_class)->finalize(obj);
}

GtkWidget *
xfbib_entry_edit_dialog_new(XfbibBibtexEntry *entry)
{
	XfbibBibtexField *field;
	XfbibBibtexValue *value;
	const gchar *bibtype;
	GList *list;
	int n, i;

	XfbibEntryEditDialog *entry_edit_dialog;
	entry_edit_dialog = g_object_new(XFBIB_TYPE_ENTRY_EDIT_DIALOG,
			"destroy-with-parent", ((GTK_DIALOG_MODAL & GTK_DIALOG_DESTROY_WITH_PARENT) != 0),
			"has-separator", ((GTK_DIALOG_MODAL & GTK_DIALOG_NO_SEPARATOR) == 0),
			"modal", ((GTK_DIALOG_MODAL & GTK_DIALOG_MODAL) != 0),
			"title", (entry == NULL) ? "Add new entry" : "Edit existing entry",
			NULL);

	if (entry != NULL) {
		bibtype = xfbib_bibtex_entry_get_bibtype(entry);
		for (n = 0; xfbib_type_constants[n].text != NULL; n++) {
			if (strcasecmp(xfbib_type_constants[n].text, bibtype) == 0) {
				gtk_combo_box_set_active(GTK_COMBO_BOX (entry_edit_dialog->type_combo_box), n);
				break;
			}
		}

		gtk_entry_set_text(GTK_ENTRY (entry_edit_dialog->key_entry), xfbib_bibtex_entry_get_key(entry));

		for (n = 0; n < XFBIB_BIBTEX_FIELD_N_FIELDS; n++) {
			list = xfbib_bibtex_entry_get_fields(entry);

			for (i = 0, value = NULL; i < g_list_length(list); i++) {
				field = g_list_nth_data(list, i);
				if (strcasecmp(xfbib_bibtex_field_get_variable(field),
							xfbib_bibtex_field_constants[n].label) == 0) {
					value = xfbib_bibtex_field_get_value(field);
					break;
				}
			}
			if (value == NULL) {
				/* Did not find a matching field */
				continue;
			}

			if (g_list_length(xfbib_bibtex_value_get(value)) == 1) {
				/* Single value */
				if (n == XFBIB_BIBTEX_FIELD_AUTHOR || n == XFBIB_BIBTEX_FIELD_EDITOR) {
					xfbib_multiple_input_set_text (XFBIB_MULTIPLE_INPUT (entry_edit_dialog->inputs[n].input),
							xfbib_bibtex_value_get_str(value));
				} else {
					gtk_entry_set_text(GTK_ENTRY (entry_edit_dialog->inputs[n].input),
							xfbib_bibtex_value_get_str(value));
				}
			} else {
				if (n == XFBIB_BIBTEX_FIELD_AUTHOR || n == XFBIB_BIBTEX_FIELD_EDITOR) {
					xfbib_multiple_input_set(XFBIB_MULTIPLE_INPUT(entry_edit_dialog->inputs[n].input),
							xfbib_bibtex_value_get(value));
				} else {
					gtk_widget_set_sensitive(entry_edit_dialog->inputs[n].input, FALSE);
					entry_edit_dialog->inputs[n].gl = xfbib_bibtex_value_get(value);
				}
			}
		}
	}

	return GTK_WIDGET(entry_edit_dialog);
}

XfbibBibtexEntry *
xfbib_entry_edit_dialog_get_entry(XfbibEntryEditDialog *entry_edit_dialog)
{
	XfbibBibtexEntry *entry;
	XfbibBibtexField *field;
	GList *field_list = NULL;
	XfbibBibtexValue *value;
	XfbibState *state;
	int n;
	const gchar *str;

	state = xfbib_state_new();

	entry = xfbib_bibtex_entry_new(G_OBJECT(xfbib_state_get_bibtex_elements(state))/* TODO: Elements */);
	xfbib_bibtex_entry_set_separator(entry, '{');	/* TODO: Should the separator be "taken" from somewhere or is it arbitrary? */
	xfbib_bibtex_entry_set_key(entry, gtk_entry_get_text(GTK_ENTRY(entry_edit_dialog->key_entry)));
	xfbib_bibtex_entry_set_bibtype(entry, gtk_combo_box_get_active_text(GTK_COMBO_BOX(entry_edit_dialog->type_combo_box)));
	for (n = 0; n < XFBIB_BIBTEX_FIELD_N_FIELDS; n++) {
		if ((n == XFBIB_BIBTEX_FIELD_AUTHOR) || (n == XFBIB_BIBTEX_FIELD_EDITOR)) {
			str = xfbib_multiple_input_get_str(XFBIB_MULTIPLE_INPUT(entry_edit_dialog->inputs[n].input));
			if(str != NULL) {
				if(strcmp(str, "") == 0) {
					continue;
				}
				field = xfbib_bibtex_field_new(G_OBJECT(entry));
				xfbib_bibtex_field_set_variable(field, xfbib_bibtex_field_constants[n].label);
				value = xfbib_bibtex_value_new(G_OBJECT(field));
				xfbib_bibtex_value_set_str(value, str);
			} else {
				field = xfbib_bibtex_field_new(G_OBJECT(entry));
				xfbib_bibtex_field_set_variable(field, xfbib_bibtex_field_constants[n].label);
				value = xfbib_bibtex_value_new(G_OBJECT(field));
				xfbib_bibtex_value_set(value, xfbib_multiple_input_get(XFBIB_MULTIPLE_INPUT(entry_edit_dialog->inputs[n].input)));
			}
			field_list = g_list_append(field_list, field);
			xfbib_bibtex_field_set_value(field, value);
		} else {
			if(GTK_WIDGET_IS_SENSITIVE(GTK_ENTRY(entry_edit_dialog->inputs[n].input))) {
				if(strcmp(gtk_entry_get_text(GTK_ENTRY(entry_edit_dialog->inputs[n].input)), "") == 0) {
					continue;
				}
				/* TODO: check if they are integers or strings(variables), don't assume text */
				field = xfbib_bibtex_field_new(G_OBJECT(entry));
				xfbib_bibtex_field_set_variable(field, xfbib_bibtex_field_constants[n].label);
				value = xfbib_bibtex_value_new(G_OBJECT(field));
				xfbib_bibtex_value_set_str(value, gtk_entry_get_text(GTK_ENTRY(entry_edit_dialog->inputs[n].input)));
				xfbib_bibtex_field_set_value(field, value);
				field_list = g_list_append(field_list, field);
			} else {
				field = xfbib_bibtex_field_new(G_OBJECT(entry));
				xfbib_bibtex_field_set_variable(field, xfbib_bibtex_field_constants[n].label);
				value = xfbib_bibtex_value_new(G_OBJECT(field));
				xfbib_bibtex_value_set(value, entry_edit_dialog->inputs[n].gl);
				xfbib_bibtex_field_set_value(field, value);
				field_list = g_list_append(field_list, field);
			}
		}
	}
	xfbib_bibtex_entry_set_fields(entry, field_list);
	return entry;
}

gboolean
xfbib_entry_edit_dialog_valid_entry(XfbibEntryEditDialog *entry_edit_dialog)
{
	if(gtk_combo_box_get_active(GTK_COMBO_BOX(entry_edit_dialog->type_combo_box)) == -1) {
		xfce_dialog_show_warning(NULL, NULL, _("A BibTeX type is required!"));
		return FALSE;
	}
	if(strcmp(gtk_entry_get_text(GTK_ENTRY(entry_edit_dialog->key_entry)), "") == 0) {
		xfce_dialog_show_warning(NULL, NULL, _("BibTeX key is required!"));
		return FALSE;
	}

	return TRUE;
}

