/* 
 * Copyright (c) 2008 Jesper Karlsson & David Gustafsson
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <gtk/gtk.h>

#include "xfbib-window.h"
#include "xfbib-menu-bar.h"
#include "xfbib-toolbar.h"
#include "xfbib-entry-tree-view.h"
#include "xfbib-string-tree-view.h"
#include "xfbib-statusbar.h"
#include "xfbib-file-io.h"

struct _XfbibWindow
{
	GtkWindow parent;
	GtkWidget *vbox;
	GtkWidget *menu_bar;
	GtkWidget *toolbar;
	GtkWidget *notebook;
	GtkWidget *entry_scrolled_window;
	GtkWidget *entry_tree_view;
	GtkWidget *string_scrolled_window;
	GtkWidget *string_tree_view;
	GtkWidget *status_bar;
};
	
typedef struct _XfbibWindowClass
{
	GtkWindowClass parent;
} XfbibWindowClass;

static void xfbib_window_class_init(XfbibWindowClass *klass);

static void xfbib_window_init(XfbibWindow *instance);
static void xfbib_window_finalize(GObject *obj);

static GObjectClass *xfbib_window_parent_class = NULL;

GType
xfbib_window_get_type (void)
{
	static GType type = 0;
	if (type == 0) {
		static const GTypeInfo info = {
			sizeof (XfbibWindowClass),
			NULL,   /* base_init */
			NULL,   /* base_finalize */
			(GClassInitFunc) xfbib_window_class_init,   /* class_init */
			NULL,   /* class_finalize */
			NULL,   /* class_data */
			sizeof (XfbibWindow),
			0,      /* n_preallocs */
			(GInstanceInitFunc) xfbib_window_init    /* instance_init */
		};
		type = g_type_register_static (GTK_TYPE_WINDOW, "XfbibWindow", &info, 0);
	}
	return type;
}

	
static void
xfbib_window_class_init(XfbibWindowClass *klass)
{
	GObjectClass *object_class = (GObjectClass *)klass;
	
	xfbib_window_parent_class = g_type_class_peek_parent (klass); 

	object_class->finalize = xfbib_window_finalize;
}

static void
xfbib_window_init(XfbibWindow *instance)
{
	instance->vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(instance), instance->vbox);
	
	instance->menu_bar = xfbib_menu_bar_new(instance);
	gtk_box_pack_start(GTK_BOX(instance->vbox), instance->menu_bar, FALSE, FALSE, 0);

	instance->toolbar = xfbib_toolbar_new(instance);
	gtk_box_pack_start(GTK_BOX(instance->vbox), instance->toolbar, FALSE, FALSE, 0);
	

	instance->notebook = gtk_notebook_new ();
	gtk_container_set_border_width(GTK_CONTAINER (instance->notebook), 0);
	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (instance->notebook), GTK_POS_TOP);
	gtk_box_pack_start(GTK_BOX(instance->vbox), instance->notebook, TRUE , TRUE, 0);


	instance->entry_scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (instance->entry_scrolled_window),
			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_notebook_append_page (GTK_NOTEBOOK (instance->notebook), instance->entry_scrolled_window, gtk_label_new(_("Entry")));

	instance->entry_tree_view = xfbib_entry_tree_view_new();
	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(instance->entry_scrolled_window), instance->entry_tree_view);
	
	instance->string_scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (instance->string_scrolled_window),
			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_notebook_append_page (GTK_NOTEBOOK (instance->notebook), instance->string_scrolled_window, gtk_label_new(_("Variable")));

	instance->string_tree_view = xfbib_string_tree_view_new();
	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(instance->string_scrolled_window), instance->string_tree_view);
	

	instance->status_bar = xfbib_statusbar_new();
	gtk_box_pack_end(GTK_BOX(instance->vbox), instance->status_bar, FALSE, FALSE, 0);

	gtk_window_set_icon_name(GTK_WINDOW(instance), "xfbib");
}

static void
xfbib_window_finalize(GObject *obj)
{
	G_OBJECT_CLASS(xfbib_window_parent_class)->finalize(obj);
}

GtkWidget *
xfbib_window_new()
{
	XfbibWindow *window;
	window = g_object_new(XFBIB_TYPE_WINDOW, NULL);
	return GTK_WIDGET(window);
}

void
xfbib_window_set_title(XfbibWindow *window, gchar *name)
{
	char *str;
	g_return_if_fail(XFBIB_IS_WINDOW(window));
	str = g_strdup_printf("%s - %s", (name == NULL) ? _("(Untitled)") : name, PACKAGE_NAME);
	gtk_window_set_title(GTK_WINDOW(window), str);
	g_free(str);
}

GtkWidget *
xfbib_window_get_tree_view(XfbibWindow *window, enum XFBIB_BIBTEX_TYPE type)
{
	g_return_val_if_fail(XFBIB_IS_WINDOW(window), NULL);
	if (type == TYPE_ENTRY)
		return window->entry_tree_view;
	else if (type == TYPE_STRING)
		return window->string_tree_view;
	return NULL;
}
